#include "filtre.h"
#include "mainwindow.h"
#include <math.h>
#include <qstring.h>


/*!
 * \brief initFiltre initialise une structure sFiltre
 * \param pFiltre est un pointeur sur la structure à initialiser
 * \param nbInitB est le nombre de coefficients bi
 * \param pBInit est un pointeur sur le tableau des coefficients bi
 * \param nbInitA est le nombre de coefficients aj
 * \param pAInit est un pointeur sur le tableau des coefficients aj
 */
void initFiltre(sFiltre * pFiltre, int nbInitB, double *pBInit, int nbInitA, double *pAInit)
{
    pFiltre->nbCoeffB = nbInitB;
    pFiltre->pCoeffB = pBInit;
    pFiltre->nbCoeffA = nbInitA;
    pFiltre->pCoeffA = pAInit;
}



/*!
 * \brief calculReponseFiltre range dans le tableau pointé par sk la réponse du filtre pFiltre à l'entrée ek
 * \param pFiltre est un pointeur sur le filtre utilisé
 * \param ek est un pointeur sur le tableau des échantillons ek
 * \param sk est un pointeur sur le tableau des échantillons sk où est rangé la réponse du filtre
 * \param nbPts est le nombre de points qui seront calculés, en partant de l'indice 0.
 */
void calculReponseFiltre(sFiltre * pFiltre, double * ek, double * sk, int nbPts)
{
    for (int k=0; k < nbPts; k++)
    {
        sk[k] = 0;
        for (int i=0; i < pFiltre->nbCoeffB; i++)
            if (k>=i)
                sk[k] = sk[k] + pFiltre->pCoeffB[i] * ek[k-i];
        for (int j=1; j < pFiltre->nbCoeffA; j++)
            if (k>=j)
                sk[k] = sk[k] - pFiltre->pCoeffA[j] * sk[k-j];
        if (pFiltre->pCoeffA[0] != 0)
            sk[k] = sk[k] / pFiltre->pCoeffA[0] ;
    }
}



/*!
 * \brief calculModule range dans le tableau pModule le module du filtre pFiltre calculé aux fréquences indiquées par pFreq
 * \param pFiltre est un pointeur sur le filtre utilisé
 * \param pFreq est un pointeur sur le tableau contenant les fréquences où sera calculé le module
 * \param nbFreq est la taille du tableau pFreq
 * \param fEch est la fréquence d'echantillonnage
 * \param db permet de calculer le module (false) ou le module en db (true)
 * \param pModule est un pointeur sur le tableau où seront rangés les valeurs calculées
 */
void calculModule(sFiltre * pFiltre, double * pFreq, int nbFreq, double fEch, bool db, double * pModule)
{
    double reNum, imNum, reDen, imDen;
    for (int f=0; f<nbFreq; f++)
    {
        reNum=0; imNum=0; reDen=0; imDen=0;
        for(int i=0; i<pFiltre->nbCoeffB; i++)
        {
            reNum += pFiltre->pCoeffB[i]*cos(2*M_PI*pFreq[f]*i/fEch);
            imNum += pFiltre->pCoeffB[i]*sin(-2*M_PI*pFreq[f]*i/fEch);
        }
        for(int j=0; j<pFiltre->nbCoeffA; j++)
        {
            reDen += pFiltre->pCoeffA[j]*cos(2*M_PI*pFreq[f]*j/fEch);
            imDen += pFiltre->pCoeffA[j]*sin(-2*M_PI*pFreq[f]*j/fEch);
        }
    if ( db )
            pModule[f] = 20* log10( sqrt( (pow(reNum,2)+pow(imNum,2))/(pow(reDen,2)+pow(imDen,2)) ));
    else
            pModule[f] = sqrt( (pow(reNum,2)+pow(imNum,2))/((pow(reDen,2)+pow(imDen,2))) );
    }
}



/*!
 * \brief calculPhase range dans le tableau pPhase la phase du filtre pFiltre calculé aux fréquences indiquées par pFreq
 * \param pFiltre est un pointeur sur le filtre utilisé
 * \param pFreq est un pointeur sur le tableau contenant les fréquences où sera calculé la phase
 * \param nbFreq est la taille du tableau pFreq
 * \param fEch est la fréquence d'echantillonnage
 * \param pPhase est un pointeur sur le tableau où seront rangés les valeurs calculées
 */
void calculPhase(sFiltre * pFiltre, double * pFreq, int nbPts, double fEch, double * pPhase)
{
    double reNum, imNum, reDen, imDen;
    for (int f=0; f<nbPts; f++)
    {
        reNum=0; imNum=0; reDen=0; imDen=0;
        for(int i=0; i<pFiltre->nbCoeffB; i++)
        {
            reNum += pFiltre->pCoeffB[i]*cos(-2*M_PI*pFreq[f]*i/fEch);
            imNum += pFiltre->pCoeffB[i]*sin(-2*M_PI*pFreq[f]*i/fEch);
        }
        for(int j=0; j<pFiltre->nbCoeffA; j++)
        {
            reDen += pFiltre->pCoeffA[j]*cos(-2*M_PI*pFreq[f]*j/fEch);
            imDen += pFiltre->pCoeffA[j]*sin(-2*M_PI*pFreq[f]*j/fEch);
        }
        pPhase[f] = atan2(imNum*reDen-reNum*imDen,reNum*reDen+imNum*imDen);
    }
}



/*!
 * \brief initDataATracer initialise une structure DataATracer
 * \param data est un pointeur sur la structure à initialiser
 * \param nInit est le nombre de points à tracer
 * \param pXInit est un pointeur sur le tableau des valeurs de l'axe X
 * \param pYInit est un pointeur sur le tableau des valeurs de l'axe Y
 */
void initDataATracer( DataATracer * data, int nInit, double *pXInit, double *pYInit)
{
    data->nbData=nInit;
    data->pX=pXInit;
    data->pY=pYInit;
}

/*!
 * \brief calculModuleAnalogique range dans le tableau pModule le module du filtre pFiltre calculé aux fréquences indiquées par pFreq
 * \param pFiltre est un pointeur sur le filtre utilisé, les coefficients doivent être rangés selon les puissances croissantes de p
 * \param pFreq est un pointeur sur le tableau contenant les fréquences où sera calculé le module
 * \param nbFreq est la taille du tableau pFreq
 * \param fEch est la fréquence d'echantillonnage
 * \param db permet de calculer le module (false) ou le module en db (true)
 * \param pModule est un pointeur sur le tableau où seront rangés les valeurs calculées
 */
void calculModuleAnalogique(sFiltre * pFiltre, double * pFreq, int nbFreq, bool db, double * pModule)
{
   double reNum, imNum, reDen, imDen;
   for (int f=0; f<nbFreq; f++)
   {
       reNum=pFiltre->pCoeffB[0]; imNum=0; reDen=pFiltre->pCoeffA[0]; imDen=0;
       for(int i=1; i<pFiltre->nbCoeffB; i++)
       {
           if ( (i % 2) == 0)
               reNum -= pFiltre->pCoeffB[i]*pow(-1,i/2)*pow(2*M_PI*pFreq[f],i);
           else
               imNum += pFiltre->pCoeffB[i]*pow(-1,i/2)*pow(2*M_PI*pFreq[f],i);
       }
       for(int i=1; i<pFiltre->nbCoeffA; i++)
       {
           if ( (i % 2) == 0)
               reDen -= pFiltre->pCoeffA[i]*pow(-1,i/2)*pow(2*M_PI*pFreq[f],i);
           else
               imDen += pFiltre->pCoeffA[i]*pow(-1,i/2)*pow(2*M_PI*pFreq[f],i);
       }
   if ( db )
           pModule[f] = 20* log10( sqrt( (pow(reNum,2)+pow(imNum,2))/(pow(reDen,2)+pow(imDen,2)) ));
   else
           pModule[f] = sqrt( (pow(reNum,2)+pow(imNum,2))/((pow(reDen,2)+pow(imDen,2))) );
   }
}

Filtre::Filtre()
{
}

